	SAVE
	LISTING OFF
	
;Calibration code for EasyPDK programmer.
;For use with asl <http://john.ccac.rwth-aachen.de:8000/as/>.
;Reverse engineered and assembly macros created by Arne Rossius, 2022-01-06.
;This file must be included *after* the device specific include file!

;The calibration macro should be inserted once in the main program, after
;selecting the desired clock source and divider (i.e. after writing CLKMD).

;calibrate high frequency R/C oscillator
EASYPDK_CALIBRATE_IHRC MACRO freq, vcc
	and	a, 'R' ;magic
	and	a, 'C' ;magic
	and	a, 1 ;1 for IHRC
	and	a, freq     & 0xFF;desired frequency (in Hz), 32 bit little endian
	and	a, freq>> 8 & 0xFF
	and	a, freq>>16 & 0xFF
	and	a, freq>>24 & 0xFF
	and	a, vcc    & 0xFF ;VCC for calibration (in mV), 16 bit little endian
	and	a, vcc>>8 & 0xFF
	and	a, #IHRCR ;register for calibration value
	ENDM

;calibrate low frequency R/C oscillator
EASYPDK_CALIBRATE_ILRC MACRO freq, vcc
	and	a, 'R' ;magic
	and	a, 'C' ;magic
	and	a, 2 ;2 for ILRC
	and	a, freq     & 0xFF ;desired frequency (in Hz), 32 bit little endian
	and	a, freq>> 8 & 0xFF
	and	a, freq>>16 & 0xFF
	and	a, freq>>24 & 0xFF
	and	a, vcc    & 0xFF ;VCC for calibration (in mV), 16 bit little endian
	and	a, vcc>>8 & 0xFF
	and	a, #ILRCR ;register for calibration value
	ENDM

;calibrate bandgap reference
EASYPDK_CALIBRATE_BG MACRO
	and	a, 'B' ;magic
	and	a, 'G' ;magic
	and	a, 3 ;3 for BG
	and	a, 3950 & 0xFF ;???
	and	a, 3950 >> 8 ;???
	and	a, #BGTR ;register for calibration value?
	and	a, #GPCC ;comparator control register
	and	a, #GPCS ;comparator selection register
	and	a, #0 ;dummy?
	and	a, #0 ;dummy?
	and	a, #0 ;dummy?
	and	a, #0 ;dummy?
	and	a, #0 ;dummy?
	and	a, #0 ;dummy?
	ENDM



;The dummy code in these macros will be replaced by the calibration code by the
;EasyPDK programmer. After calibration, most of the calibration code will be
;overwritten with NOP instructions, leaving only two instructions: a MOV to
;load the calibration value into the accumulator, and another MOV to store the
;accumulator to the calibration register (IHRCR, ILRCR or BGTR).
;
;Oscillator calibration:
;
;calibration code substituted by programmer (PFS154, easypdkprog 1.3):
;	set1	PAC, 3		;set PA3 to output
;	set1	PADIER, 4	;enable PA4 digital input
;	mov	a, #0xFF	;first calibration value = 0xFF ...
;	add	a, #1		;... plus 1 = 0x00
;loop:	set1	PA, 3		;set PA3 output high
;	mov	IHRCR/ILRCR, a	;apply calibration value
;	t0sn	PA, 4		;if PA4 input is high ...
;	add	a, #1		;... increment calibration value
;	set0	PA, 3		;set PA3 output low
;	goto	loop		;loop indefinitely
;code after calibration complete (OTP can only change bits from 1 to 0):
;	nop			;opcode 0x0000, i.e. all bits 0
;	nop
;	mov	a, #CAL_VALUE	;previously 0xFF, i.e. all bits 1
;	nop
;	nop
;	mov	IHRCR/ILRCR, a	;no change
;	nop
;	nop
;	nop
;	nop
;
;Bandgap calibration:
;
;calibration code substituted by programmer (PFS154, easypdkprog 1.3):
;	mov	a, #0x09	;set PA3 & PA0 to output
;	mov	PAC, a
;	set1	PADIER, 4	;enable PA4 digital input
;	mov	a, #0x84	;enable comparator, (-) = bandgap, (+) = VIR
;	mov	GPCC, a
;	mov	a, #0xA3	;comparator output to PA0, VIR = Vcc * 1.2/2.8
;	mov	GPCS, a
;	mov	a, #0xFF	;first calibration value = 0xFF
;loop:	set1	PA, 3		;set PA3 output high
;	mov	BGTR, a		;apply calibration value
;	t0sn	PA, 4		;if PA input is high ...
;	add	a, #1		;... increment calibration value
;	set0	PA, 3		;set PA3 output low
;	goto	loop		;loop indefinitely
;code after calibration complete (OTP can only change bits from 1 to 0):
;	nop			;opcode 0x0000, i.e. all bits 0
;	nop
;	nop
;	nop
;	nop
;	nop
;	nop
;	mov	a, #CAL_VALUE	;previously 0xFF, i.e. all bits 1
;	nop
;	mov	BGTR, a		;no change
;	nop
;	nop
;	nop
;	nop

	RESTORE
